define(
[
    'jquery',
    'underscore',
    'backbone'
],

function($, _, Backbone){
'use strict';

/**
 * @name Async
 * @description an Async model, uses jQuery deferreds
 * @returns {Backbone.Model}
 * @requires jquery, underscore, backbone
 */

var Async = Backbone.Model.extend({

    options: null,
    promise: null,
    deferred: null,

    url: function() {
        return this.options.url;
    },


    // Initialization

    /**
     * @name initialize
     * @description initialize this object
     * @param  {object} options
     *         {
     *             url: '/path/to/resource',
     *             done: callback(model, response, options){ ... },
     *             fail: callback(model, response, options){ ... }
     *         }
     * @return nothing
     */
    initialize: function(options) {
        this.options = options || {};

        if(!this.options.url) {
            throw new Error('Async model requires a resource location!');
        }

        // proxy funcction scopes
        _.bindAll(
            this, // desired scope
            'onFetchSuccess',
            'onFetchError'
        );

        // setup jquery deferred
        this.deferred = $.Deferred();
        this.promise = this.deferred.promise();

        // fetch immediately
        this.fetch(
            {
                success: this.onFetchSuccess,
                error: this.onFetchError
            }
        );
    },


    // Methods

    done: function(callback) {
        this.promise.done(callback);
    },

    fail: function(callback) {
        this.promise.fail(callback);
    },


    // Fetch callbacks

    onFetchSuccess: function(model, response, options) {
        // console.log('fetch succeeded!', model, response, options);

        this.deferred.resolve(model, response, options, this.promise);

        if(_.isFunction(this.options.done)) {
            this.deferred.done(this.options.done);
        }
    },

    onFetchError: function(model, response, options) {
        // console.log('fetch encountered an error!', model, response, options);

        this.deferred.reject(model, response, options, this.promise);

        if(_.isFunction(this.options.fail)) {
            this.deferred.fail(this.options.fail);
        }
    }

});

return Async;

});
